package ch.nostromo.tiffanys.clients.desktop.gui.gamesetup;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.ButtonGroup;

import ch.nostromo.lib.controller.NosResourcesException;
import ch.nostromo.lib.controller.NosSettingsException;
import ch.nostromo.lib.swing.NosButton;
import ch.nostromo.lib.swing.NosLabel;
import ch.nostromo.lib.swing.NosPanel;
import ch.nostromo.lib.swing.NosTextField;
import ch.nostromo.lib.swing.NosToggleButton;
import ch.nostromo.tiffanys.clients.desktop.controller.DesktopController;
import ch.nostromo.tiffanys.clients.desktop.gui.board.BoardPanel;
import ch.nostromo.tiffanys.clients.desktop.gui.settings.EngineSettingsPanel;
import ch.nostromo.tiffanys.game.GameController;
import ch.nostromo.tiffanys.game.GameControllerException;
import ch.nostromo.tiffanys.game.player.Player;
import ch.nostromo.tiffanys.game.player.PlayerEngine;
import ch.nostromo.tiffanys.game.player.PlayerException;
import ch.nostromo.tiffanys.game.player.PlayerHuman;
import ch.nostromo.tiffanys.game.player.PlayerTools;

public class GameSetupPanel extends NosPanel {

  private static final long serialVersionUID = 1L;

  private static Logger logger = Logger.getLogger(GameSetupPanel.class.getName());

  private GameController currentGameController = null;
  private NosLabel lblPlayerWhite;
  private NosButton btnWhiteSettings;
  private BoardPanel boardPanel;
  private ButtonGroup blackButtonGroup;
  private NosButton btnFen;
  private NosButton nosButton1;
  private NosButton btnStartNewGame;
  private ButtonGroup whiteButtonGroup;
  private NosButton btnBlackSettings;
  private NosTextField txtPlayerBlackName;
  private NosToggleButton btnBlackEngine;
  private NosToggleButton btnBlackHuman;
  private NosLabel lblPlayerBlack;
  private NosTextField txtFen;
  private NosTextField txtPlayerWhiteName;
  private NosToggleButton btnWhiteEngine;
  private NosToggleButton btnWhiteHuman;

  public GameSetupPanel(GameController gameController, boolean newGame) throws PlayerException, NosResourcesException, CloneNotSupportedException {
    initGUI();
    initNOS(gameController, newGame);
  }

  public GameSetupPanel() {
    initGUI();
  }

  private void initGUI() {

    {
      this.setPreferredSize(new java.awt.Dimension(588, 388));
      this.setLayout(null);
      this.setSize(588, 388);
      {
        lblPlayerWhite = new NosLabel();
        this.add(lblPlayerWhite);
        lblPlayerWhite.setText("Player White");
        lblPlayerWhite.setBounds(9, 285, 93, 15);
        lblPlayerWhite.setToolTipTranslate(true);
        lblPlayerWhite.setTextTranslationKey("Setup.PlayerWhite");
      }
      {
        btnWhiteHuman = new NosToggleButton();
        this.add(btnWhiteHuman);
        btnWhiteHuman.setText("H");
        btnWhiteHuman.setBounds(108, 282, 50, 22);
      }
      {
        btnWhiteEngine = new NosToggleButton();
        this.add(btnWhiteEngine);
        btnWhiteEngine.setText("T");
        btnWhiteEngine.setBounds(164, 282, 50, 22);
      }
      {
        txtPlayerWhiteName = new NosTextField();
        this.add(txtPlayerWhiteName);
        txtPlayerWhiteName.setBounds(226, 282, 196, 22);
      }
      {
        btnWhiteSettings = new NosButton();
        this.add(btnWhiteSettings);
        btnWhiteSettings.setText("Engine Settings");
        btnWhiteSettings.setBounds(428, 282, 147, 22);
        btnWhiteSettings.setTextTranslationKey("Setup.EngineSettings");
        btnWhiteSettings.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {
            whiteSettingsAction();
          }
        });
      }
      {
        boardPanel = new BoardPanel();
        this.add(boardPanel);
        boardPanel.setBounds(152, 54, 177, 178);
      }
      {
        txtFen = new NosTextField();
        this.add(txtFen);
        txtFen.setBounds(109, 244, 467, 22);
      }
      {
        lblPlayerBlack = new NosLabel();
        this.add(lblPlayerBlack);
        lblPlayerBlack.setText("Player Black");
        lblPlayerBlack.setBounds(9, 21, 93, 15);
        lblPlayerBlack.setTextTranslationKey("Setup.PlayerBlack");
      }
      {
        btnBlackHuman = new NosToggleButton();
        this.add(btnBlackHuman);
        btnBlackHuman.setText("H");
        btnBlackHuman.setBounds(108, 18, 50, 22);
      }
      {
        btnBlackEngine = new NosToggleButton();
        this.add(btnBlackEngine);
        btnBlackEngine.setText("T");
        btnBlackEngine.setBounds(164, 18, 50, 22);
      }
      {
        txtPlayerBlackName = new NosTextField();
        this.add(txtPlayerBlackName);
        txtPlayerBlackName.setBounds(226, 18, 196, 22);
      }
      {
        btnBlackSettings = new NosButton();
        this.add(btnBlackSettings);
        btnBlackSettings.setText("Engine Settings");
        btnBlackSettings.setBounds(428, 18, 147, 22);
        btnBlackSettings.setToolTipTranslate(true);
        btnBlackSettings.setTextTranslationKey("Setup.EngineSettings");
        btnBlackSettings.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {
            blackSettingsAction();
          }
        });
      }
      {
        btnStartNewGame = new NosButton();
        this.add(btnStartNewGame);
        btnStartNewGame.setText("New game");
        btnStartNewGame.setBounds(108, 325, 161, 29);
        btnStartNewGame.setTextTranslate(false);
        btnStartNewGame.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {
            startGameAction();
          }
        });
      }
      {
        nosButton1 = new NosButton();
        this.add(nosButton1);
        nosButton1.setText("Abort");
        nosButton1.setBounds(414, 325, 161, 29);
        nosButton1.setTextTranslationKey("Common.Cancel");
        nosButton1.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {
            closeAction();
          }
        });
      }
      {
        btnFen = new NosButton();
        this.add(btnFen);
        btnFen.setText("FEN");
        btnFen.setBounds(9, 244, 74, 22);
        btnFen.setTextTranslationKey("Setup.Fen");
        btnFen.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {
            fenAction();
          }
        });
      }
      this.blackButtonGroup = new ButtonGroup();
      this.whiteButtonGroup = new ButtonGroup();

      this.blackButtonGroup.add(this.btnBlackEngine);
      btnBlackEngine.setTextTranslationKey("Setup.Engine");
      btnBlackEngine.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          switchBlackToEngine();
        }
      });
      this.blackButtonGroup.add(this.btnBlackHuman);
      btnBlackHuman.setToolTipTranslate(true);
      btnBlackHuman.setTextTranslationKey("Setup.Human");
      btnBlackHuman.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          switchBlackToHuman();
        }
      });

      this.whiteButtonGroup.add(this.btnWhiteEngine);
      btnWhiteEngine.setTextTranslationKey("Setup.Engine");
      btnWhiteEngine.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          switchWhiteToEngine();
        }
      });
      this.whiteButtonGroup.add(this.btnWhiteHuman);
      btnWhiteHuman.setTextTranslationKey("Setup.Human");
      btnWhiteHuman.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          switchWhiteToHuman();
        }
      });

    }

  }

  public void initNOS(GameController gameController, boolean newGame) throws PlayerException, NosResourcesException, CloneNotSupportedException {

    // Update board gfx
    this.remove(boardPanel);
    this.boardPanel = new BoardPanel(null);
    this.add(boardPanel);
    boardPanel.setBounds(108, 54, 177, 178);

    this.currentGameController = gameController.clone();

    if (newGame) {
      this.btnFen.setEnabled(true);
      this.btnStartNewGame.setText(DesktopController.getInstance().getTranslation("Setup.NewGame"));
    } else {
      this.btnFen.setEnabled(false);
      this.btnStartNewGame.setText(DesktopController.getInstance().getTranslation("Setup.ContinueGame"));
    }

    this.btnFen.setEnabled(newGame);

    // Player white
    Player white = gameController.getPlayerWhite();
    this.txtPlayerWhiteName.setText(white.getPlayerName());

    // Player black
    Player black = gameController.getPlayerBlack();
    this.txtPlayerBlackName.setText(black.getPlayerName());

    this.setInputStatus();

    this.txtFen.setText(gameController.getFEN());
    this.boardPanel.setBoard(gameController.getCurrentBoard());

  }

  private void setInputStatus() throws PlayerException {

    // Player white
    Player white = currentGameController.getPlayerWhite();
    this.txtPlayerWhiteName.setText(white.getPlayerName());
    if (white instanceof PlayerEngine) {
      this.btnWhiteSettings.setEnabled(true);
      this.txtPlayerWhiteName.setEnabled(false);
      this.btnWhiteEngine.setSelected(true);
      this.btnWhiteHuman.setSelected(false);
    } else {
      this.btnWhiteSettings.setEnabled(false);
      this.txtPlayerWhiteName.setEnabled(true);
      this.btnWhiteEngine.setSelected(false);
      this.btnWhiteHuman.setSelected(true);
    }

    Player black = currentGameController.getPlayerBlack();
    this.txtPlayerBlackName.setText(black.getPlayerName());
    if (black instanceof PlayerEngine) {
      this.btnBlackSettings.setEnabled(true);
      this.txtPlayerBlackName.setEnabled(false);
      this.btnBlackEngine.setSelected(true);
      this.btnBlackHuman.setSelected(false);
    } else {
      this.btnBlackSettings.setEnabled(false);
      this.txtPlayerBlackName.setEnabled(true);
      this.btnBlackEngine.setSelected(false);
      this.btnBlackHuman.setSelected(true);
    }

  }

  void switchWhiteToEngine() {
    try {
      currentGameController.setPlayerWhite(DesktopController.getInstance().getDefaultPlayerEngine());
      this.setInputStatus();
    } catch (PlayerException e) {
      logger.log(Level.SEVERE, "Player exception", e);
    } catch (NosSettingsException e) {
      logger.log(Level.SEVERE, "Settings exception", e);
    }
  }

  void switchWhiteToHuman() {
    try {
      currentGameController.setPlayerWhite(DesktopController.getInstance().getDefaultPlayerHuman());
      this.setInputStatus();
    } catch (PlayerException e) {
      logger.log(Level.SEVERE, "Player exception", e);
    } catch (NosResourcesException e) {
      logger.log(Level.SEVERE, "Resources exception", e);
    }
  }

  void switchBlackToEngine() {
    try {
      currentGameController.setPlayerBlack(DesktopController.getInstance().getDefaultPlayerEngine());
      this.setInputStatus();
    } catch (PlayerException e) {
      logger.log(Level.SEVERE, "Player exception", e);
    } catch (NosSettingsException e) {
      logger.log(Level.SEVERE, "Settings exception", e);
    }
  }

  void switchBlackToHuman() {
    try {
      currentGameController.setPlayerBlack(DesktopController.getInstance().getDefaultPlayerHuman());
      this.setInputStatus();
    } catch (PlayerException e) {
      logger.log(Level.SEVERE, "Player exception", e);
    } catch (NosResourcesException e) {
      logger.log(Level.SEVERE, "Resources exception", e);
    }

  }

  void closeAction() {
    this.nosClose();
  }

  void whiteSettingsAction() {
    try {
      EngineSettingsPanel esp = new EngineSettingsPanel(((PlayerEngine) currentGameController.getPlayerWhite()).getEngineSettings());
      String title = DesktopController.getInstance().getTranslation("EngineSettings.Title");
      DesktopController.getInstance().getDesktop().showPanelAsDialog(esp, title);
    } catch (NosResourcesException e) {
      logger.log(Level.SEVERE, "Resources exception", e);
    }
  }

  void blackSettingsAction() {
    try {
      EngineSettingsPanel esp = new EngineSettingsPanel(((PlayerEngine) currentGameController.getPlayerBlack()).getEngineSettings());
      String title = DesktopController.getInstance().getTranslation("EngineSettings.Title");
      DesktopController.getInstance().getDesktop().showPanelAsDialog(esp, title);
    } catch (NosResourcesException e) {
      logger.log(Level.SEVERE, "Resources exception", e);
    }
  }

  void fenAction() {
    try {
      String fen = txtFen.getText();
      Player white = currentGameController.getPlayerWhite();
      Player black = currentGameController.getPlayerBlack();
      currentGameController = new GameController(white, black, fen);
      this.boardPanel.setBoard(currentGameController.getCurrentBoard());
    } catch (GameControllerException e) {
      logger.log(Level.SEVERE, "GameControler exception", e);
    }
  }

  void startGameAction() {
    // Update Players Names if Human
    if (currentGameController.getPlayerWhite() instanceof PlayerHuman) {
      currentGameController.setPlayerWhite(PlayerTools.createPlayerHuman(txtPlayerWhiteName.getText()));
    }

    if (currentGameController.getPlayerBlack() instanceof PlayerHuman) {
      currentGameController.setPlayerBlack(PlayerTools.createPlayerHuman(txtPlayerBlackName.getText()));
    }

    DesktopController.getInstance().launchNewGame(currentGameController);

    this.closeAction();
  }

}
