package ch.nostromo.tiffanys.clients.xboard.xbif;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import ch.nostromo.lib.controller.NosSettingsException;
import ch.nostromo.tiffanys.clients.xboard.controller.XBoardController;
import ch.nostromo.tiffanys.engines.EngineSettings;
import ch.nostromo.tiffanys.game.GameController;
import ch.nostromo.tiffanys.game.GameControllerException;
import ch.nostromo.tiffanys.game.GameControllerListener;
import ch.nostromo.tiffanys.game.GameControllerResult;
import ch.nostromo.tiffanys.game.game.Game;
import ch.nostromo.tiffanys.game.game.GameIllegalMoveException;
import ch.nostromo.tiffanys.game.game.GameState;
import ch.nostromo.tiffanys.game.move.MoveInput;
import ch.nostromo.tiffanys.game.player.Player;
import ch.nostromo.tiffanys.game.player.PlayerTools;

public class XBoardInterface implements GameControllerListener {
  private static Logger logger = Logger.getLogger(XBoardInterface.class.getName());

  private GameController gameController = null;

  private int currentEngineColor = Game.COLOR_BLACK;
  
  public XBoardInterface() {
    // Do nothing

  }

  private void doOutput(String msg) {
    logger.info("Sending to xboard -> " + msg);
    System.out.println(msg);
  }

  public void playGame() throws NosSettingsException, IOException  {

    String settingsFile = XBoardController.getInstance().getSettings().ENGINE_SETTINGS;
    EngineSettings engineSettings = new EngineSettings(settingsFile);

    Player playerWhite = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, engineSettings);
    Player playerBlack = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, engineSettings);

    BufferedReader keyb = new BufferedReader(new InputStreamReader(System.in));
    doOutput(XBoardController.getInstance().getSettings().XBOARD_FEATURES);

    while (true) {
      String cmd = keyb.readLine();
      logger.info("Getting from XBoard <- " + cmd);

      try {

        // Filter commands
        if (cmd.startsWith("quit")) {
          break;
        } else if (cmd.startsWith("xboard")) {
          // Do nothing

        } else if (cmd.startsWith("protover")) {

          // Do nothing
        } else if (cmd.startsWith("accepted")) {
          // Do nothing

        } else if (cmd.startsWith("new")) {

          logger.fine("Creating new game controller!");

          gameController = new GameController(playerWhite, playerBlack);
          gameController.addGameControllerListener(this);

          currentEngineColor = Game.COLOR_BLACK;
          
        } else if (cmd.startsWith("random")) {
          // Do nothing

        } else if (cmd.startsWith("level")) {

          setLevel(cmd);

        } else if (cmd.startsWith("sd")) {
          
          setSd(cmd);

        } else if (cmd.startsWith("st")) {
          
          setSt(cmd);
          
        } else if (cmd.startsWith("time")) {
          // Do nothing

        } else if (cmd.startsWith("otim")) {
          // Do nothing

        } else if (cmd.startsWith("hard")) {
          // Do nothing

        } else if (cmd.startsWith("white")) {
          // Do nothing

        } else if (cmd.startsWith("black")) {
          // Do nothing

        } else if (cmd.startsWith("force")) {
          // Do nothing

        } else if (cmd.startsWith("result")) {
          // Do nothing

        } else if (cmd.startsWith("post")) {
          // Do nothing

        } else if (cmd.startsWith("nopost")) {
          // Do nothing

        } else if (cmd.startsWith("easy")) {
          // Do nothing

        } else if (cmd.startsWith("edit")) {
          // Do nothing

        } else if (cmd.startsWith("#")) {
          // Do nothing

        } else if (cmd.startsWith("computer")) {
          // Do nothing

        } else if (cmd.startsWith("go")) {

          logger.fine("Go received.");
          if (currentEngineColor == gameController.getCurrentColorToMove()) {
            logger.fine("Go ignored, as allready engine turn");
          } else {
            logger.fine("Compute next move");
            gameController.computeNextMove(true);
          }
        } else {

          logger.fine("Move recieved, entering move: " + cmd);
          // Append xboard move to our game
          MoveInput fromXBoard = XBoardMove.xBoard2Tiff(cmd);
          gameController.enterNextMove(fromXBoard);
          
          currentEngineColor = gameController.getCurrentColorToMove();
          

          // Check if game is over, if not, reply with new move
          GameState gameState = gameController.getCurrentGameState();
          if (!gameState.gameOpen()) {
            finishGame(gameState);
          } else {
            logger.fine("Compute next move.");
            gameController.computeNextMove(true);
          }

        }
      } catch (GameIllegalMoveException err) {
        // doOutput("tellusererror MESSAGE: Illegal move: " + cmd);
        // doOutput("Illegal move: " + cmd);
        logger.log(Level.INFO, "IllegalMoveException: ", err);
      } catch (XBoardMoveParsingException err) {
        // doOutput("tellusererror MESSAGE:" + err.getMessage());
        // doOutput("Error:" + err.getMessage());
        logger.log(Level.INFO, "XBoardMoveParsingException: ", err);
      } catch (GameControllerException err) {
        // doOutput("tellusererror MESSAGE:" + err.getMessage());
        // doOutput("Error: " + err.getMessage());
        logger.log(Level.SEVERE, "Unexpected Error: ", err);
      }
    }

  }

  private void finishGame(GameState gameState) {

    if (gameState.whiteMates()) {
      logger.fine("Game finished. White mates!");
      doOutput("1-0 {White mates}");
    } else if (gameState.blackMates()) {
      logger.fine("Game finished. Black mates!");
      doOutput("0-1 {Black mates}");
    } else if (gameState.remisByMaterial()) {
      logger.fine("Game finished. Remis by material!");
      doOutput("1/2-1/2 {Draw by material}");
    } else if (gameState.remisByFifty()) {
      logger.fine("Game finished. Remis by 50 moves rule!");
      doOutput("1/2-1/2 {Draw by repetition 50}");
    } else if (gameState.remisByThree()) {
      logger.fine("Game finished. Remis by 3 times board repetition!");
      doOutput("1/2-1/2 {Draw by position repetition 3}");
    } else if (gameState.remisByStaleMate()) {
      logger.fine("Game finished. Remis by stale mate!");
      doOutput("1/2-1/2 {Draw by Stale Mate}");
    }

  }

  public void computeNextLegalMovesFinished(GameControllerResult gameControllerResult) {
    // TODO Auto-generated method stub

  }

  public void computeNextMoveFinished(GameControllerResult gameControllerResult) {
    try {
      logger.fine("Next move computed: " + gameControllerResult.getSelectedMove().getSanDescription());
      doOutput("move " + XBoardMove.tiff2XBoard(gameControllerResult.getSelectedMove()));

      GameState gameState = gameController.getCurrentGameState();

      if (!gameState.gameOpen()) {
        finishGame(gameState);
      }
    } catch (GameControllerException e) {
      logger.log(Level.SEVERE, "Unexpected error", e);
    }
  }

  public void computeNextMoveDepthFinished(GameControllerResult engineResult) {
    // TODO Auto-generated method stub

  }
  
  private void setLevel(String cmd) {
    logger.fine("Parsing level");
    StringTokenizer st = new StringTokenizer(cmd);
    
    st.nextToken(); // Level
    
    String mps = st.nextToken();
    String base = st.nextToken();
    st.nextToken(); // Inc
    
    int movesCount = Integer.valueOf(mps).intValue();
    int seconds = getSeconds(base);
    
    EngineSettings newSettings = new EngineSettings();
    newSettings.GAME_MODE = EngineSettings.GAME_MODE_FIXED_TIME_PER_MOVES;
    newSettings.FIXED_TIME_PER_MOVES_MOVE_COUNT = movesCount;
    newSettings.FIXED_TIME_PER_MOVES_SECONDS = seconds;
    
    logger.fine("Setting engine to " + movesCount + " moves in " + seconds + " seconds");
    
    Player playerWhite = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, newSettings);
    Player playerBlack = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, newSettings);

    gameController.setPlayerWhite(playerWhite);
    gameController.setPlayerBlack(playerBlack);
    
  }

  private int getSeconds(String base) {
    int dp = base.indexOf(":");
    String minutes = "";
    String seconds = "";
    
    if (dp >= 0) {
      minutes = base.substring(0, dp -1);
      seconds = base.substring(dp + 1);
    } else {
      minutes = base;
    }
    int min = 0;
    if (minutes.length() > 0) {
      min = Integer.valueOf(minutes).intValue();
    }
    
    int sec = 0;
    if (seconds.length() > 0) {
      sec = Integer.valueOf(seconds).intValue();
    }

    return (min * 60) + sec;
      
  }

  private void setSd(String cmd) {
    int depth = Integer.valueOf(cmd).intValue();

    
    EngineSettings newSettings = new EngineSettings();
    newSettings.GAME_MODE = EngineSettings.GAME_MODE_FIXED_DEPTH;
    newSettings.FIXED_DEPTH = depth;
    
    logger.fine("Setting engine to depth "  + depth);
    
    Player playerWhite = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, newSettings);
    Player playerBlack = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, newSettings);

    gameController.setPlayerWhite(playerWhite);
    gameController.setPlayerBlack(playerBlack);
    
  }
  
  private void setSt(String cmd) {
    
    int sec = getSeconds(cmd);
    
    EngineSettings newSettings = new EngineSettings();
    newSettings.GAME_MODE = EngineSettings.GAME_MODE_FIXED_TIME_PER_MOVE;
    newSettings.FIXED_TIME_PER_MOVE = sec;
    
    logger.fine("Setting engine to " + sec + " seconds per move");
    
    Player playerWhite = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, newSettings);
    Player playerBlack = PlayerTools.createPlayerEngine(XBoardController.getInstance().getSettings().ENGINE, newSettings);

    gameController.setPlayerWhite(playerWhite);
    gameController.setPlayerBlack(playerBlack);
    
  }
}
