package ch.nostromo.tiffanys.engines;

import java.util.Enumeration;
import java.util.Vector;

import ch.nostromo.tiffanys.game.board.Board;

public abstract class Engine extends Thread {

  public static final int MAX_RANGE = 99999;

  protected static final int MAX_MATE_DEPTH = 999;

  public static final int MAX_MATE_RANGE = MAX_RANGE - MAX_MATE_DEPTH;
  
  private Vector<EngineListener> listener;

  private Board board;

  private int colorToMove;
  
  protected EngineSettings engineSettings = null;
  
  protected boolean running = true;
  
  public abstract EngineResult engineSelectNextMove(Board parBoard, int parColorToMove);

  public abstract String getEngineName();
    
  EngineResult engineResult = null;
  
  public void setEngineSearchParameters(Board board, int colorToMove) {
    this.board = board;
    this.colorToMove = colorToMove;
  }
  
  public void startEngine() {
    // Check for timing
    
    if (engineSettings.GAME_MODE == EngineSettings.GAME_MODE_FIXED_TIME_PER_MOVES) {
      double secPerMove = engineSettings.FIXED_TIME_PER_MOVES_SECONDS /  engineSettings.FIXED_TIME_PER_MOVES_MOVE_COUNT;
      secPerMove -= engineSettings.TIME_HOLE;
      EngineTimer et = new EngineTimer(this, secPerMove);
      et.start();
      start();
    } else if (engineSettings.GAME_MODE == EngineSettings.GAME_MODE_FIXED_TIME_PER_MOVE) {
      double secPerMove = engineSettings.FIXED_TIME_PER_MOVE;
      secPerMove -= engineSettings.TIME_HOLE;
      EngineTimer et = new EngineTimer(this, secPerMove);
      et.start();
      start();
    } else {
      start();
    }
  }

  /** Run method. A bit complicated, cause suspend() and resume() are depricated */
  @Override
  public void run() {
    running = true;
    engineResult = engineSelectNextMove(board, colorToMove);
    processEngineFinishedEvent(engineResult);
  }

  public synchronized void addEngineListener(EngineListener m) {
    if (listener == null) {
      listener = new Vector<EngineListener>(5, 5);
    }
    if (!listener.contains(m)) {
      listener.addElement(m);
    }
  }

  private synchronized void processEngineFinishedEvent(EngineResult engineResult) {
    if (listener != null) {
      for (Enumeration<EngineListener> e = listener.elements(); e.hasMoreElements();) {
        (e.nextElement()).engineFinished(engineResult);
      }
    }
  }
  protected synchronized void processDepthFinished(EngineResult engineResult) {
    if (listener != null) {
      for (Enumeration<EngineListener> e = listener.elements(); e.hasMoreElements();) {
        (e.nextElement()).depthFinished(engineResult);
      }
    }
  }
  
  public void setEngineSettings(EngineSettings engineSettings) {
    this.engineSettings = engineSettings;
  }
 
  public EngineSettings getEngineSettings() {
    return engineSettings;
  }

  public boolean isRunning() {
    return running;
  }

  public void setRunning(boolean running) {
    this.running = running;
  }

  public EngineResult getEngineResult() {
    return engineResult;
  }


}
