package ch.nostromo.tiffanys.engines.tiffanys.board;

import java.util.ArrayList;

import ch.nostromo.lib.util.NosTools;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysBishop;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysKing;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysKnight;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysPawn;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysPiece;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysQueen;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysRook;
import ch.nostromo.tiffanys.game.board.Board;

public class TiffanysBoard {

  public static double stats_position_checK = 0;

  public static double stats_checked_check = 0;

  public static final int IDX_WHITE_KING = 121;

  public static final int IDX_BLACK_KING = 122;

  public static final int IDX_WHITE_MATERIAL = 123;

  public static final int IDX_BLACK_MATERIAL = 124;

  // Most important Board values
  private static int[] mainBoard = new int[125];

  public TiffanysBoard(Board board) {
    for (int i = 0; i < board.getBoardArray().length; i++) {
      mainBoard[i] = board.getBoardArray()[i];
    }
    setKingPositions(mainBoard);
    setMaterial(mainBoard);
  }

  public int[] getBoard() {
    return mainBoard;
  }

  private void setMaterial(int[] board) {
    board[IDX_WHITE_MATERIAL] = 0;
    board[IDX_BLACK_MATERIAL] = 0;
    for (int i = 21; i <= 98; i++) {
      if (NosTools.checkMask(board[i], Board.PIECE_QUEEN)) {
        if (NosTools.checkMask(board[i], Board.OWNER_WHITE)) {
          board[IDX_WHITE_MATERIAL] += TiffanysQueen.MATERIAL_VALUE;
        } else {
          board[IDX_BLACK_MATERIAL] += TiffanysQueen.MATERIAL_VALUE;
        }
      }
      if (NosTools.checkMask(board[i], Board.PIECE_ROOK)) {
        if (NosTools.checkMask(board[i], Board.OWNER_WHITE)) {
          board[IDX_WHITE_MATERIAL] += TiffanysRook.MATERIAL_VALUE;
        } else {
          board[IDX_BLACK_MATERIAL] += TiffanysRook.MATERIAL_VALUE;
        }
      }
      if (NosTools.checkMask(board[i], Board.PIECE_KNIGHT)) {
        if (NosTools.checkMask(board[i], Board.OWNER_WHITE)) {
          board[IDX_WHITE_MATERIAL] += TiffanysKnight.MATERIAL_VALUE;
        } else {
          board[IDX_BLACK_MATERIAL] += TiffanysKnight.MATERIAL_VALUE;
        }
      }
      if (NosTools.checkMask(board[i], Board.PIECE_BISHOP)) {
        if (NosTools.checkMask(board[i], Board.OWNER_WHITE)) {
          board[IDX_WHITE_MATERIAL] += TiffanysBishop.MATERIAL_VALUE;
        } else {
          board[IDX_BLACK_MATERIAL] += TiffanysBishop.MATERIAL_VALUE;
        }
      }
      if (NosTools.checkMask(board[i], Board.PIECE_PAWN)) {
        if (NosTools.checkMask(board[i], Board.OWNER_WHITE)) {
          board[IDX_WHITE_MATERIAL] += TiffanysPawn.MATERIAL_VALUE;
        } else {
          board[IDX_BLACK_MATERIAL] += TiffanysPawn.MATERIAL_VALUE;
        }
      }
    }
  }

  private void setKingPositions(int[] board) {
    for (int i = 21; i <= 98; i++) {
      if (NosTools.checkMask(board[i], Board.PIECE_KING)) {
        if (NosTools.checkMask(board[i], Board.OWNER_WHITE)) {
          board[IDX_WHITE_KING] = i;
        } else {
          board[IDX_BLACK_KING] = i;
        }
      }
    }
  }

  // ------------------------- STATIC Board functions -------------------------

  public static final int invertColor(int color) {
    if (color == Board.OWNER_WHITE) {
      return Board.OWNER_BLACK;
    } else {
      return Board.OWNER_WHITE;
    }
  }

  public static final int getOwner(int field) {
    if (NosTools.checkMask(field, Board.OWNER_WHITE))
      return Board.OWNER_WHITE;
    if (NosTools.checkMask(field, Board.OWNER_BLACK))
      return Board.OWNER_BLACK;
    return Board.EMPTY_FIELD;
  }

  public static final boolean isEnemyPiece(int field, int myColor) {
    return (getOwner(field) == invertColor(myColor));
  }

  public static final boolean isEmptyField(int field) {
    return NosTools.checkMask(field, Board.EMPTY_FIELD);
  }

  public static final boolean isMovedField(int field) {
    return NosTools.checkMask(field, Board.MOVED);
  }

  public static final boolean isValidMoveToField(int field, int myColor) {
    return (isEmptyField(field) || isEnemyPiece(field, myColor));
  }

  public static final TiffanysMove[] generateMoveList(int[] board, int color) {
    TiffanysMove[] moves = new TiffanysMove[100];
    int movesIdx = 0;
    for (int i = 21; i <= 98; i++) {
      if ((color & board[i]) != 0) {
        if ((Board.PIECE_KNIGHT & board[i]) != 0) {
          movesIdx = TiffanysKnight.getMoveList(board, i, color, movesIdx, moves);
        } else if ((Board.PIECE_PAWN & board[i]) != 0) {
          movesIdx = TiffanysPawn.getMoveList(board, i, color, movesIdx, moves);
        } else if ((Board.PIECE_KING & board[i]) != 0) {
          movesIdx = TiffanysKing.getMoveList(board, i, color, movesIdx, moves);
        } else if ((Board.PIECE_BISHOP & board[i]) != 0) {
          movesIdx = TiffanysBishop.getMoveList(board, i, color, movesIdx, moves);
        } else if ((Board.PIECE_QUEEN & board[i]) != 0) {
          movesIdx = TiffanysQueen.getMoveList(board, i, color, movesIdx, moves);
        } else if ((Board.PIECE_ROOK & board[i]) != 0) {
          movesIdx = TiffanysRook.getMoveList(board, i, color, movesIdx, moves);
        }
      }
    }
    return moves;
  }

  public static TiffanysMove[] generateLegalMovesList(int[] board, int color) {
    ArrayList<TiffanysMove> result = new ArrayList<TiffanysMove>();

    
    TiffanysMove[] moves = generateMoveList(board, color);

    for (int i = 0; i < moves.length; i++) {
      if (moves[i] != null) {
        TiffanysBoard.makeMove(board, moves[i], color);
        boolean isCheck = checkForCheck(board, color);
        TiffanysBoard.takeBackMove(board, moves[i]);
        if (!isCheck) {
          result.add(moves[i]);
        }
      }
    }

    return result.toArray(new TiffanysMove[result.size()]);
  }

  public static void takeBackMove(int[] board, TiffanysMove move) {
    board[move.from] = move.backupFrom;
    board[move.to] = move.backupTo;
    board[IDX_BLACK_KING] = move.backupBlackKing;
    board[IDX_BLACK_MATERIAL] = move.backupBlackMaterial;
    board[IDX_WHITE_KING] = move.backupWhiteKing;
    board[IDX_WHITE_MATERIAL] = move.backupWhiteMaterial;

    board[Board.IDX_ENPASSANT_FIELD] = move.backupEpField;
    if (move.enPassantMove) {
      board[move.enPassantOpponentTake] = move.backupEpTakenField;
    }

    // Backup castling fields
    if (move.castling == TiffanysMove.CASTLING_LONG) {
      board[move.castlingRook] = move.backupCastlingRook;
      board[move.castlingRook + 3] = move.backupCastlingRookTo;
    } else if (move.castling == TiffanysMove.CASTLING_SHORT) {
      board[move.castlingRook] = move.backupCastlingRook;
      board[move.castlingRook - 2] = move.backupCastlingRookTo;
    }

  }

  public static boolean checkForCheck(int[] board, int currentColor) {
    stats_checked_check++;
    if (currentColor == Board.OWNER_WHITE) {
      return TiffanysPiece.isKingAttacked(board, board[IDX_WHITE_KING], currentColor);
    } else {
      return TiffanysPiece.isKingAttacked(board, board[IDX_BLACK_KING], currentColor);
    }
  }

  public static void makeMove(int[] board, TiffanysMove move, int color) {

    // Now backup the board fields for later take back of the move
    move.backupFrom = board[move.from];
    move.backupTo = board[move.to];
    move.backupBlackKing = board[IDX_BLACK_KING];
    move.backupWhiteKing = board[IDX_WHITE_KING];
    move.backupWhiteMaterial = board[IDX_WHITE_MATERIAL];
    move.backupBlackMaterial = board[IDX_BLACK_MATERIAL];
    move.backupEpField = board[Board.IDX_ENPASSANT_FIELD];

    if (move.enPassantMove) {
      move.backupEpTakenField = board[move.enPassantOpponentTake];
    }

    // Backup castling fields
    if (move.castling == TiffanysMove.CASTLING_LONG) {
      move.backupCastlingRook = board[move.castlingRook];
      move.backupCastlingRookTo = board[move.castlingRook + 3];
    } else if (move.castling == TiffanysMove.CASTLING_SHORT) {
      move.backupCastlingRook = board[move.castlingRook];
      move.backupCastlingRookTo = board[move.castlingRook - 2];
    }

    int enemyColor = invertColor(color);
    int enemyMaterialIdx = IDX_WHITE_MATERIAL;
    int ownMaterialIdx = IDX_BLACK_MATERIAL;
    if (enemyColor == Board.OWNER_BLACK) {
      enemyMaterialIdx = IDX_BLACK_MATERIAL;
      ownMaterialIdx = IDX_WHITE_MATERIAL;
    }

    // Castling first
    if (move.castling != TiffanysMove.CASTLING_NO) {

      if (color == Board.OWNER_WHITE) {
        board[IDX_WHITE_KING] = move.to;
      } else if (color == Board.OWNER_BLACK) {
        board[IDX_BLACK_KING] = move.to;
      }

      board[move.to] = Board.BOARD_LEGAL + color + Board.PIECE_KING + Board.MOVED;
      board[move.from] = Board.BOARD_LEGAL + Board.EMPTY_FIELD;

      if (move.castling == TiffanysMove.CASTLING_LONG) {
        board[move.castlingRook + 3] = board[move.castlingRook];
        board[move.castlingRook] = Board.BOARD_LEGAL + Board.EMPTY_FIELD;
      } else {
        board[move.castlingRook - 2] = board[move.castlingRook];
        board[move.castlingRook] = Board.BOARD_LEGAL + Board.EMPTY_FIELD;
      }

    } else {

      // Calculate new material if enemy takes a hit
      if (NosTools.checkMask(board[move.to], enemyColor)) {
        if (NosTools.checkMask(board[move.to], Board.PIECE_QUEEN)) {
          board[enemyMaterialIdx] -= TiffanysQueen.MATERIAL_VALUE;
        } else if (NosTools.checkMask(board[move.to], Board.PIECE_ROOK)) {
          board[enemyMaterialIdx] -= TiffanysRook.MATERIAL_VALUE;
        } else if (NosTools.checkMask(board[move.to], Board.PIECE_BISHOP)) {
          board[enemyMaterialIdx] -= TiffanysBishop.MATERIAL_VALUE;
        } else if (NosTools.checkMask(board[move.to], Board.PIECE_KNIGHT)) {
          board[enemyMaterialIdx] -= TiffanysKnight.MATERIAL_VALUE;
        } else if (NosTools.checkMask(board[move.to], Board.PIECE_PAWN)) {
          board[enemyMaterialIdx] -= TiffanysPawn.MATERIAL_VALUE;
        }
        move.hitmove = true;
      }
      
      // Calculate new material if promotion
      if (move.startingPiece != move.endingPiece) {
        if (move.endingPiece == Board.PIECE_BISHOP) {
          board[ownMaterialIdx] += TiffanysBishop.MATERIAL_VALUE;
          board[ownMaterialIdx] -= TiffanysPawn.MATERIAL_VALUE;
        } 
        if (move.endingPiece == Board.PIECE_ROOK) {
          board[ownMaterialIdx] += TiffanysRook.MATERIAL_VALUE;
          board[ownMaterialIdx] -= TiffanysPawn.MATERIAL_VALUE;
        } 
        if (move.endingPiece == Board.PIECE_KNIGHT) {
          board[ownMaterialIdx] += TiffanysKnight.MATERIAL_VALUE;
          board[ownMaterialIdx] -= TiffanysPawn.MATERIAL_VALUE;
        } 
        if (move.endingPiece == Board.PIECE_QUEEN) {
          board[ownMaterialIdx] += TiffanysQueen.MATERIAL_VALUE;
          board[ownMaterialIdx] -= TiffanysPawn.MATERIAL_VALUE;
        } 
        
      }
      
      // If king is moved, save his position
      if (move.startingPiece == Board.PIECE_KING && color == Board.OWNER_WHITE) {
        board[IDX_WHITE_KING] = move.to;
      } else if (move.startingPiece == Board.PIECE_KING && color == Board.OWNER_BLACK) {
        board[IDX_BLACK_KING] = move.to;
      }

      // Remove piece
      board[move.from] = Board.BOARD_LEGAL + Board.EMPTY_FIELD;

      // Move piece
      if (move.startingPiece == move.endingPiece) {
        board[move.to] = Board.BOARD_LEGAL + color + move.startingPiece + Board.MOVED;
      } else {
        board[move.to] = Board.BOARD_LEGAL + color + move.endingPiece + Board.MOVED;
      }

      // Remove en passant pawn if needed
      if (move.enPassantMove) {
        if (color == Board.OWNER_WHITE) {
          board[move.to - 10] = Board.BOARD_LEGAL + Board.EMPTY_FIELD;
        } else {
          board[move.to + 10] = Board.BOARD_LEGAL + Board.EMPTY_FIELD;
        }
        move.hitmove = true;
      }

      // Set or remove enPassant field
      if (move.leavingEnPassantField > 0) {
        board[Board.IDX_ENPASSANT_FIELD] = move.leavingEnPassantField;
      } else {
        board[Board.IDX_ENPASSANT_FIELD] = -1;
      }

    }
  }

}
