package ch.nostromo.tiffanys.engines.tiffanys.board;

import ch.nostromo.tiffanys.game.board.BoardTools;
import ch.nostromo.tiffanys.game.move.MoveInput;

public class TiffanysMove implements Comparable<TiffanysMove> {

  public static final int CASTLING_NO = 0;

  public static final int CASTLING_SHORT = 1;

  public static final int CASTLING_LONG = 2;

  public int from = -1;

  public int to = -1;

  public int startingPiece = -1;

  public int endingPiece = -1;

  public boolean enPassantMove = false;
  
  public int enPassantOpponentTake = -1;

  public int leavingEnPassantField = -1;

  public int castling = CASTLING_NO;

  public int castlingRook = -1;

  public int score = 0;

  public int scoreDepth = 0;
  
  public int backupEpField = -1;
  public int backupFrom = -1;
  public int backupTo = -1;
  public int backupEpTakenField = 0;
  public int backupWhiteKing = -1;
  public int backupBlackKing = -1;
  public int backupWhiteMaterial = -1;
  public int backupBlackMaterial = -1;
  public int backupCastlingRook = -1;
  public int backupCastlingRookTo = -1;

  

  public TiffanysMove[] scoredMoveTree = null;

  public boolean hitmove;

  public TiffanysMove(int from, int to, int startingPiece, int endingPiece, int leavingEnPassantField) {
    this.from = from;
    this.to = to;
    this.startingPiece = startingPiece;
    this.endingPiece = endingPiece;
    this.leavingEnPassantField = leavingEnPassantField;
  }

  public TiffanysMove(int from, int to, int startingPiece, int endingPiece) {
    this(from, to, startingPiece, endingPiece, -1);
  }

  public TiffanysMove(int from, int to, int startingPiece) {
    this(from, to, startingPiece, startingPiece);
  }

  private static MoveInput smallConv(TiffanysMove move) {
    String fromS = BoardTools.fieldToCoord(move.from);
    String toS = BoardTools.fieldToCoord(move.to);

    MoveInput result = null;
    if (move.endingPiece == move.startingPiece) {
      result = new MoveInput(fromS, toS);
    } else {
      result = new MoveInput(fromS, toS, move.endingPiece);
    }
    return result;
  }
  
  public MoveInput convertToMoveInput() {
    String fromS = BoardTools.fieldToCoord(from);
    String toS = BoardTools.fieldToCoord(to);

    MoveInput result = null;
    if (endingPiece == startingPiece) {
      result = new MoveInput(fromS, toS);
    } else {
      result = new MoveInput(fromS, toS, endingPiece);
    }
    // Score
    result.setScore(score);
    result.setScoreDepth(scoreDepth);

    // Move Tree
    if (scoredMoveTree != null) {
      MoveInput[] scoreTree = new MoveInput[scoredMoveTree.length];
      for (int i = 0; i < scoreTree.length; i++) {
        if (scoredMoveTree[i] != null) {
          scoreTree[i] = smallConv( scoredMoveTree[i]);
        }
      }
      result.setScoreTree(scoreTree);
    }
    
    return result;
  }

  /**
   * Implements the comparable interface
   * 
   * @param o
   *          The object to compare
   * @return Standard compare values
   */
  public int compareTo(TiffanysMove o) {
    if (o == null)
      return 1;
    TiffanysMove tmp = o;
    if (tmp.score > score) {
      return 1;
    }
    if (tmp.score < score) {
      return -1;
    }
    return 0;

  }

}