package ch.nostromo.tiffanys.engines.tiffanys.board.pieces;

import ch.nostromo.tiffanys.engines.tiffanys.board.TiffanysBoard;
import ch.nostromo.tiffanys.engines.tiffanys.board.TiffanysMove;
import ch.nostromo.tiffanys.engines.tiffanys.board.pieces.TiffanysPiece;
import ch.nostromo.tiffanys.game.board.Board;

public class TiffanysPawn extends TiffanysPiece {

  private static final int myPieceCode = Board.PIECE_PAWN;

  public static final int MATERIAL_VALUE = 100;

  public static int getMoveList(int[] board, int currentPosition, int myColor, int oldArrayLength, TiffanysMove[] moves) {
    int arrayLength = oldArrayLength;
    int direction = 1;

    if (myColor == Board.OWNER_BLACK)
      direction = -1;

    // One step forward (including possible "pawn promotion")
    int toField = currentPosition + (10 * direction);
    if (TiffanysBoard.isEmptyField(board[toField])) {
      if (toField > 90 || toField < 29) {
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_KNIGHT);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_QUEEN);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_ROOK);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_BISHOP);
        arrayLength++;
      } else {
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode);
        arrayLength++;
      }
    }

    // Two steps forward
    int toField1 = toField;
    toField = currentPosition + (20 * direction);
    if (!TiffanysBoard.isMovedField(board[currentPosition]) && TiffanysBoard.isEmptyField(board[toField1]) && TiffanysBoard.isEmptyField(board[toField])) {
      moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, myPieceCode, toField1);
      arrayLength++;
    }

    // hit Moves ...
    toField = currentPosition + (11 * direction);
    if (LEGALBOARD[toField] && TiffanysBoard.isEnemyPiece(board[toField], myColor)) {
      if (toField > 90 || toField < 29) {
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_KNIGHT);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_QUEEN);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_ROOK);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_BISHOP);
        arrayLength++;
      } else {
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode);
        arrayLength++;
      }
    }

    // also for enpassant
    if (toField == board[Board.IDX_ENPASSANT_FIELD]) {
      moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode);
      moves[arrayLength].enPassantMove = true;
      moves[arrayLength].enPassantOpponentTake = toField + (direction * -10);
      arrayLength++;
    }

    // hit Moves to the other side
    toField = currentPosition + (9 * direction);
    if (LEGALBOARD[toField] && TiffanysBoard.isEnemyPiece(board[toField], myColor)) {
      if (toField > 90 || toField < 29) {
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_KNIGHT);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_QUEEN);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_ROOK);
        arrayLength++;
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode, Board.PIECE_BISHOP);
        arrayLength++;
      } else {
        moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode);
        arrayLength++;
      }

    }
    // also for enpassant
    if (toField == board[Board.IDX_ENPASSANT_FIELD]) {
      moves[arrayLength] = new TiffanysMove(currentPosition, toField, myPieceCode);
      moves[arrayLength].enPassantMove = true;
      moves[arrayLength].enPassantOpponentTake = toField + (direction * -10);
      arrayLength++;
    }

    return arrayLength;

  }


}