package ch.nostromo.tiffanys.engines.tiffanys.board.pieces;

import ch.nostromo.lib.util.NosTools;
import ch.nostromo.tiffanys.engines.tiffanys.board.TiffanysBoard;
import ch.nostromo.tiffanys.game.board.Board;

public abstract class TiffanysPiece {

  public static boolean[] LEGALBOARD = new boolean[120];

  public static boolean initialized = false;

  public static void initialize() {
    if (initialized) {
      return;
    }

    initializeLegalBoard();
  }

  private static void initializeLegalBoard() {

    // First, all fields are "legal"
    for (int i = 0; i < 120; i++) {
      LEGALBOARD[i] = true;
    }
    // Overwrite some fields as "void"
    for (int i = 0; i <= 19; i++) {
      LEGALBOARD[i] = false;
    }
    for (int i = 20; i <= 90; i += 10) {
      LEGALBOARD[i] = false;
    }
    for (int i = 29; i <= 99; i += 10) {
      LEGALBOARD[i] = false;
    }
    for (int i = 100; i <= 119; i++) {
      LEGALBOARD[i] = false;
    }

  }

  public static short checkDirection(int board[], int myPosition, int myColor, int direction) {
    short result = 0;
    while (true) {
      int toField = myPosition + ((result + 1) * direction);
      if (!LEGALBOARD[toField]) {
        return result;
      }

      if ((Board.EMPTY_FIELD & board[toField]) != 0) {
        result++;
      } else {
        if (TiffanysBoard.isEnemyPiece(board[toField], myColor)) {
          result++;
          return result;
        } else {
          return result;
        }
      }

      // boolean isEnemyPiece = TiffanysBoard.isEnemyPiece(board[toField],
      // myColor);
      // if ((Board.EMPTY_FIELD & board[toField]) != 0 || isEnemyPiece) {
      // result ++;
      // if (isEnemyPiece) {
      // return result;
      // }
      // } else {
      // return result;
      // }
    }
  }

  public static short checkEnemiesInDirection(int board[], int myPosition, int enemyColor, int direction) {
    short result = 0;
    while (true) {
      int toField = myPosition + ((result + 1) * direction);
      if (!LEGALBOARD[toField]) {
        return result;
      }

      if ((Board.EMPTY_FIELD & board[toField]) != 0) {
        result++;
      } else {
        if ((enemyColor & board[toField]) != 0) {
          result++;
          return result;
        } else {
          return 8;
        }
      }

    }
  }

  public static boolean isKingAttacked(int[] board, int pos, int currentColor) {
    int enemyColor = TiffanysBoard.invertColor(currentColor);
    int fieldContentToCheck = 0;
    // Check for Knight Attacks
    if (NosTools.checkMask(board[pos + 19], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos + 19], enemyColor))
      return true;
    if (NosTools.checkMask(board[pos + 21], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos + 21], enemyColor))
      return true;
    if (NosTools.checkMask(board[pos + 12], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos + 12], enemyColor))
      return true;
    if (NosTools.checkMask(board[pos - 8], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos - 8], enemyColor))
      return true;
    if (NosTools.checkMask(board[pos - 19], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos - 19], enemyColor))
      return true;
    if (NosTools.checkMask(board[pos - 21], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos - 21], enemyColor))
      return true;
    if (NosTools.checkMask(board[pos - 12], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos - 12], enemyColor))
      return true;
    if (NosTools.checkMask(board[pos + 8], Board.PIECE_KNIGHT) && NosTools.checkMask(board[pos + 8], enemyColor))
      return true;

    // Check for diagonal Attacks
    int dcount = checkEnemiesInDirection(board, pos, enemyColor, 11);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * 11)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_BISHOP) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING)) {
      return true;
    }
      
    dcount = checkEnemiesInDirection(board, pos, enemyColor, -11);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * -11)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_BISHOP) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING)) {
      return true;
    }
    
    dcount = checkEnemiesInDirection(board, pos, enemyColor, 9);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * 9)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_BISHOP) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING)) {
      return true;
    }

    dcount = checkEnemiesInDirection(board, pos, enemyColor, -9);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * -9)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_BISHOP) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING)) {
      return true;
    }

    dcount = checkEnemiesInDirection(board, pos, enemyColor, 1);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * 1)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_ROOK) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING)) {
      return true;
    }
    
    dcount = checkEnemiesInDirection(board, pos, enemyColor, -1);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * -1)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_ROOK) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING))
      return true;

    dcount = checkEnemiesInDirection(board, pos, enemyColor, 10);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * 10)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_ROOK) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING)) {
      return true;
    }

    dcount = checkEnemiesInDirection(board, pos, enemyColor, -10);
    if (dcount > 0 && dcount < 8) {
      fieldContentToCheck = board[pos + (dcount * -10)];
      if (NosTools.checkMask(fieldContentToCheck, enemyColor) && ((NosTools.checkMask(fieldContentToCheck, Board.PIECE_ROOK) || (NosTools.checkMask(fieldContentToCheck, Board.PIECE_QUEEN)))))
        return true;
    }
    if (dcount == 1 && NosTools.checkMask(fieldContentToCheck, Board.PIECE_KING)) {
      return true;
    }
      

    // Check for pawn Attacks
    int pawnDirection = 1;
    if (currentColor == Board.OWNER_BLACK)
      pawnDirection = -1;

    if (NosTools.checkMask(board[pos + (11 * pawnDirection)], Board.PIECE_PAWN) && NosTools.checkMask(board[pos + (11 * pawnDirection)], enemyColor)) {
      return true;
    }

    if (NosTools.checkMask(board[pos + (9 * pawnDirection)], Board.PIECE_PAWN) && NosTools.checkMask(board[pos + (9 * pawnDirection)], enemyColor)) {
      return true;
    }

    return false;

  }

  /*
   * public static int[] getTestGame() { int[] board = new int[124];
   * Board.clearBoard(board);
   *  // board[21] += Board.OWNER_WHITE + Board.PIECE_ROOK; // board[92] +=
   * Board.OWNER_WHITE + Board.PIECE_ROOK; board[21] += Board.OWNER_WHITE +
   * Board.PIECE_KING; board[28] += Board.OWNER_BLACK + Board.PIECE_QUEEN;
   * board[38] += Board.OWNER_BLACK + Board.PIECE_QUEEN;
   * 
   * Board.setEmptyFields(board); Board.setMaterial(board);
   * 
   * 
   * return board; }
   */

}